#
# spec file for package google-compute-engine
#
# Copyright (c) 2019 SUSE LINUX GmbH, Nuernberg, Germany.
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


%define setup_version 2.8.13

Name:           google-compute-engine
Version:        20190416
Release:        lp150.2.19.1
Summary:        GCE Instance Initialization
License:        Apache-2.0
Group:          System/Daemons
Url:            https://github.com/GoogleCloudPlatform/compute-image-packages
Source0:        compute-image-packages-%{version}.tar.gz
Source7:        google-compute-engine-rpmlintrc
Source8:        baselibs.conf
Source9:        google-optimize-local-ssd.service
Source10:       google-set-multiqueue.service
Patch1:         gcei-hide-py-deps.patch
Patch2:         gcei-scripts-after-reg.patch
Patch3:         gcei-set-run_dir.patch
BuildRoot:      %{_tmppath}/%{name}-%{version}-build

%description
Google Compute Engine instance code.

%package init
Summary:        GCE Instance Initialization
Group:          System/Daemons
Requires:       google-compute-engine-oslogin == %{version}
%if 0%{?suse_version} && 0%{?suse_version} > 1315
Requires:       python3-distro
Requires:       python3-setuptools
%else
Requires:       python-setuptools
%endif
Requires:       systemd
%{?systemd_requires}
%if 0%{?suse_version} && 0%{?suse_version} > 1315
BuildRequires:  python3-distro
BuildRequires:  python3-setuptools
%else
BuildRequires:  python-setuptools
%endif
BuildArch:      noarch
%if 0%{?suse_version} && 0%{?suse_version} <= 1315
# These packages were not released in SLE 15 and later, thus no conflict needed
Conflicts:      google-startup-scripts
Conflicts:      google-daemon
Conflicts:      gcimagebundle
%endif

%description init
Initialization code for Google Compute Engine instances.

%package oslogin
Summary:        OS Login Functionality for Google Compute Engine
Group:          Hardware

Requires:       pam
Requires(post): openssh
Requires(post): glibc
%if 0%{?suse_version} && 0%{?suse_version} <= 1315
BuildRequires:  boost-devel
%endif
BuildRequires:  gcc-c++
BuildRequires:  libcurl-devel
BuildRequires:  libjson-c-devel
BuildRequires:  make
BuildRequires:  pam-devel

%description oslogin
Libraries and scripts  to enable OS Login functionality for
Google Compute Engine. Modifies sshd, nsswitch, and sshd_pam configurations.


%prep
%setup -q -n compute-image-packages-%{version}
%patch1 -p1
%patch2 -p1
%patch3 -p1
find -name "*.py" | xargs sed -i 'sm#!/usr/bin/pythonmm'
cp %{SOURCE9} google-optimize-local-ssd.service
cp %{SOURCE10} google-set-multiqueue.service

%build
cd packages/python-google-compute-engine
%if 0%{?suse_version} && 0%{?suse_version} > 1315
python3 setup.py build
%else
python setup.py build
%endif
cd ..
pushd google-compute-engine-oslogin
%if 0%{?suse_version} && 0%{?suse_version} > 1315
make %{?_smp_mflags}
%else
make %{?_smp_mflags} LIBS='-lcurl -ljson-c -lboost_regex'
%endif
popd

%install
cd packages/python-google-compute-engine
%if 0%{?suse_version} && 0%{?suse_version} > 1315
python3 setup.py install --prefix=%{_prefix} --root=%{buildroot}
%else
python setup.py install --prefix=%{_prefix} --root=%{buildroot}
%endif
cd ..
mkdir -p %{buildroot}%{_bindir}
cp google-compute-engine/src/usr/bin/* %{buildroot}%{_bindir}
mkdir -p %{buildroot}%{_sbindir}
# Init
mkdir -p %{buildroot}%{_unitdir}
cp ../google-optimize-local-ssd.service %{buildroot}%{_unitdir}
cp ../google-set-multiqueue.service %{buildroot}%{_unitdir}
cp google-compute-engine/src/lib/systemd/system/*.service %{buildroot}%{_unitdir}
for srv_name in %{buildroot}%{_unitdir}/*.service; do rc_name=$(basename -s '.service' $srv_name); ln -s service %{buildroot}%{_sbindir}/rc$rc_name; done
# Sysconfig
mkdir -p %{buildroot}%{_sysconfdir}/sysctl.d
cp google-compute-engine/src/etc/sysctl.d/11-gce-network-security.conf %{buildroot}%{_sysconfdir}/sysctl.d
# udev
mkdir -p %{buildroot}/usr/lib/udev/rules.d
cp google-compute-engine/src/lib/udev/rules.d/* %{buildroot}/usr/lib/udev/rules.d
# journald
mkdir -p %{buildroot}/usr/lib/systemd/journald.conf.d
cp google-compute-engine/src/usr/lib/systemd/journald.conf.d/* %{buildroot}/usr/lib/systemd/journald.conf.d
# syslog
mkdir -p %{buildroot}/%{_sysconfdir}/rsyslog.d
cp google-compute-engine/src/etc/rsyslog.d/* %{buildroot}/%{_sysconfdir}/rsyslog.d
# oslogin
pushd google-compute-engine-oslogin
make install DESTDIR=%{buildroot} NSS_INSTALL_PATH=%{_lib} PAM_INSTALL_PATH=%{_lib}/security
popd
# kernel module blacklist
mkdir -p %{buildroot}/%{_sysconfdir}/modprobe.d
cp google-compute-engine/src/etc/modprobe.d/gce-blacklist.conf  %{buildroot}/%{_sysconfdir}/modprobe.d/

%pre init
    if [ -f /usr/lib/systemd/system/google-ip-forwarding-daemon.service ]; then
        systemctl stop --no-block google-ip-forwarding-daemon
        systemctl disable google-ip-forwarding-daemon.service
    fi
    if [ -f /usr/lib/systemd/system/google-network-setup.service ]; then
        systemctl stop --no-block google-network-setup
        systemctl disable google-network-setup.service
    fi
    %service_add_pre google-accounts-daemon.service google-clock-skew-daemon.service google-instance-setup.service google-network-daemon.service google-optimize-local-ssd.service google-set-multiqueue.service google-shutdown-scripts.service google-startup-scripts.service

%post init
    %service_add_post google-accounts-daemon.service google-clock-skew-daemon.service google-instance-setup.service google-network-daemon.service google-optimize-local-ssd.service google-set-multiqueue.service google-shutdown-scripts.service google-startup-scripts.service
    if [ "$1" == "2" ] && ! [ -e /.buildenv ]; then
        # "$1" == "2" means the package is being upgraded
        # ./buildenv is only present during package builds
        systemctl enable google-network-daemon.service
        systemctl start google-network-daemon.service
    fi

%post oslogin 
/sbin/ldconfig

%preun init
    %service_del_preun -f google-accounts-daemon.service google-clock-skew-daemon.service google-instance-setup.service google-network-daemon.service google-optimize-local-ssd.service google-set-multiqueue.service google-shutdown-scripts.service google-startup-scripts.service

%postun init
    %service_del_postun -f google-accounts-daemon.service google-clock-skew-daemon.service google-instance-setup.service google-network-daemon.service google-optimize-local-ssd.service google-set-multiqueue.service google-shutdown-scripts.service google-startup-scripts.service

%postun oslogin 
/sbin/ldconfig

%files init
%defattr(0644,root,root,0755)
%doc CONTRIB.md README.md
%license LICENSE 
%if 0%{?suse_version} && 0%{?suse_version} > 1315
%dir %{python3_sitelib}/google_compute_engine
%dir %{python3_sitelib}/google_compute_engine-%{setup_version}-py%{py3_ver}.egg-info
%else
%dir %{python_sitelib}/google_compute_engine
%dir %{python_sitelib}/google_compute_engine-%{setup_version}-py%{py_ver}.egg-info
%endif
%exclude %{_bindir}/google_oslogin_control
%exclude %{_bindir}/google_authorized_keys
%exclude %{_bindir}/google_oslogin_nss_cache
%attr(0755,root,root) %{_bindir}/*
%if 0%{?suse_version} && 0%{?suse_version} > 1315
%{_sbindir}/*
%else
%attr(0755,root,root) %{_sbindir}/*
%endif
%if 0%{?suse_version} && 0%{?suse_version} > 1315
%{python3_sitelib}/google_compute_engine/*
%else
%{python_sitelib}/google_compute_engine/*
%endif
%if 0%{?suse_version} && 0%{?suse_version} > 1315
%{python3_sitelib}/google_compute_engine-%{setup_version}-py%{py3_ver}.egg-info/*
%else
%{python_sitelib}/google_compute_engine-%{setup_version}-py%{py_ver}.egg-info/*
%endif
%{_unitdir}/*.service
%config %{_sysconfdir}/sysctl.d/*
%dir /usr/lib/systemd
%dir /usr/lib/systemd/journald.conf.d
%dir /usr/lib/udev
%dir /usr/lib/udev/rules.d
/usr/lib/udev/rules.d/*
/usr/lib/systemd/journald.conf.d/*
%dir %{_sysconfdir}/rsyslog.d
%dir %{_sysconfdir}/modprobe.d
%config %{_sysconfdir}/rsyslog.d/*
%config %{_sysconfdir}/modprobe.d/gce-blacklist.conf

%files oslogin
%defattr(0644,root,root,0755)
%attr(0755,root,root) %{_bindir}/google_oslogin_control
%attr(0755,root,root) %{_bindir}/google_authorized_keys
%attr(0755,root,root) %{_bindir}/google_oslogin_nss_cache
/%{_lib}/*
%exclude /%{_lib}/udev*

%changelog
* Thu May 16 2019 John Paul Adrian Glaubitz <adrian.glaubitz@suse.com>
- Update to version 20190416 (bsc#1128392, bsc#1134179)
  + Google Compute Engine
  * FreeBSD fixes: syslog socket location and OS detection.
  * Upstart systems: only run startup scripts at boot.
  + Google Compute Engine OS Login
  * Fix pam_group ordering detection.
  * Restart cron from the OS Login control file.
  * Add PAM entry to su:account stack.
- from version 20190315
  + Google Compute Engine OS Login
  * Fix alternate challenge section for two factor authentication.
  * Fix FreeBSD compatibility issues in the control file.
- from version 20190304
  + Google Compute Engine
  * Set oom_score_adjust for google_accounts_daemon.
  + Google Compute Engine OS Login
  * Use pam_group to provide users with default groups.
  * Add compat.h to support FreeBSD.
  * Exit immediately after a two factor authentication failure.
  * Add support for Google phone prompt challenges.
- Adjust paths for new upstream directory layout in %%build and %%install
- Include systemd service file to run google_optimize_local_ssd command
  + google-optimize-local-ssd.service
- Include systemd service file to run google_set_multiqueue command
  + google-set-multiqueue.service
- Install journald configuration files into /usr/lib/systemd/journald.conf.d
- Refresh patches for new version
  + gcei-hide-py-deps.patch
  + gcei-scripts-after-reg.patch
  + gcei-set-run_dir.patch
* Wed Jan 30 2019 John Paul Adrian Glaubitz <adrian.glaubitz@suse.com>
- Update to version 20190124 (bsc#1123671, bsc#1123672)
  + Google Compute Engine
  * Fix metadata script retrieval to support Python 3.
* Mon Jan 21 2019 Robert Schweikert <rjschwei@suse.com>
- Remove dropped service from systemd setup macros (bsc#1122172)
- Drop use of restart_on_update, force service restart with  -f option
  on service_del_preun and service_del_postun
- Detect and handle removed services in pre rather than post
* Tue Dec 11 2018 John Paul Adrian Glaubitz <adrian.glaubitz@suse.com>
- Update to version 20181206 (bsc#1119029, bsc#1119110)
  + Google Compute Engine
  * Support enabling OS Login two factor authentication.
  * Improve accounts support for FreeBSD.
  + Google Compute Engine OS Login
  * Support OS Login two factor authentication (Alpha).
  * Improve SELinux support.
- from version 20181023
  + Google Compute Engine
  * Fix: Update sudoer group membership without overriding local groups.
- from version 20181018
  + Google Compute Engine
  * Fix: Remove users from sudoers group on account removal.
* Sun Nov 25 2018 Robert Schweikert <rjschwei@suse.com>
- Remove conditions for distributions older than Leap 42.3 and SLE 12
  + Delete init scripts google-accounts-daemon.suse,
    google-clock-skew-daemon.suse, google-instance-setup.suse,
    google-network-daemon.suse, google-shutdown-scripts.suse,
    google-startup-scripts.suse
- Fix build for distributions with gcc version less than 4.9
  + Add new dependency on boost
* Fri Oct 12 2018 John Paul Adrian Glaubitz <adrian.glaubitz@suse.com>
- Update to version 20181011
  + Google Compute Engine
  * Revert: Remove users from sudoers group on account removal.
- from version 20181008
  + Google Compute Engine
  * Remove users from sudoers group on account removal.
  * Remove gsutil dependency for metadata scripts.
- from version 20180905
  + Google Compute Engine
  * Remove ntp package dependency.
  * Support Debian 10 Buster.
  * Restart the network daemon if networking is restarted.
  * Prevent setup of the default ethernet interface.
  * Accounts daemon verifies username is 32 characters or less.
  + Google Compute Engine OS Login
  * Add user name validation to pam modules.
  * Return false on failed final load.
  * Support FreeBSD.
  * Support Debian 10 Buster.
- from version 20180611
  + Google Compute Engine
  * Prevent IP forwarding daemon log spam.
  * Make default shell configurable when executing metadata scripts.
  * Rename distro directory to distro_lib.
- Refresh patches for new version
  + gcei-set-run_dir.patch
* Mon Jun 18 2018 adrian.glaubitz@suse.com
- Ensure that google-ip-forwarding-daemon service and
  google-network-setup are stopped and disabled
  during upgrade
- Ensure that google-network-daemon service is enabled
  and started during upgrade
* Thu Jun 14 2018 adrian.glaubitz@suse.com
- Add patch to set run_dir to /var/run (bsc#1097378, #1097616)
  + gcei-set-run_dir.patch
- Drop deleted patch from spec file
  + gcei-lnx-distro-py3.patch
* Tue May 29 2018 rjschwei@suse.com
- Remove gcei-lnx-distro-py3.patch
  + Upstream intention is to depend on distro module from GitHub
- Add dependency on python3-distro for SLE/Leap 15 and later
* Mon May 21 2018 rjschwei@suse.com
- Add patch gcei-lnx-distro-py3.patch (bsc#1094074)
* Fri May 11 2018 adrian.glaubitz@suse.com
- Update to version 20180510 (bsc#1092214)
  + Prevent delay in configuring IP forwarding routes.
  + Improve instance setup support for FreeBSD.
- Include new google-network-daemon
  + Add google-network-daemon.service activation in
    %%pre and %%post sections
  + Add google-network-daemon.suse init script for SysV
- Stop shipping deprecated google-ip-forwarding-daemon service
  + Remove google-ip-forwarding-daemon.service activation in
    %%pre and %%post sections
  + Drop google-ip-forwarding-daemon.suse from source distribution
- Add missing association with "init" package for  %%pre, %%post,
  %%preun and %%postun sections
- Install google_oslogin_nss_cache binary into oslogin package
* Tue May  8 2018 adrian.glaubitz@suse.com
- Update to version 20180504 (bsc#1092214)
  + Create a new network daemon.
  + Refactor the IP forwarding daemon and network setup.
  + Improvements for using NSS cache in the accounts daemon.
  + Include libnss cache as part of the OS Login package.
- Refresh patches for new version:
  + gcei-scripts-after-reg.patch
* Mon Apr 16 2018 adrian.glaubitz@suse.com
- Update to version 20180227 (bsc#1066273)
  + Add distro specific logic.
  + Support SLES 11 and 12 in multi-nic setup.
  + Fix boto config documentation.
  + Add modprobe blacklist for nouveau and floppy modules.
  + Fix irqbalance conflict in Debian package.
  + Fix conflict with other applications that use curl and SSL.
- Install new kernel module blacklist into /etc/modprobe.d.
- Refresh patches for new version:
  + gcei-hide-py-deps.patch
* Fri Feb  2 2018 adrian.glaubitz@suse.com
- Update to version 20180129 (bsc#1078349, bsc#1079077)
  + Improve rsyslog daemon reset when using the dhcp exit hook.
  + The OS Login feature is generally available.
  + Change the OS Login uid restriction to allow uid 1000.
  + Close socket connections after requesting metadata.
- From version 20171213
  + Force IPv4 for Debian apt configs.
* Sun Dec  3 2017 adrian.glaubitz@suse.com
- Update to version 20171129 (bsc#1070895, bsc#1070918)
  + Generate SSH host keys when none are present.
  + Improve logging when activating OS Login.
  + Fix parsing logic for expiration time on SSH public keys.
  + Fix home directory creation PAM config.
* Fri Nov  3 2017 rjschwei@suse.com
- Change dependencies -init depends on -oslogin
  + oslogin feature is now enabled by the initialization code
    when appropriate
- Do not start the oslogin feature upon package install
* Fri Oct 27 2017 rjschwei@suse.com
- Fix build for SLES 11
* Thu Oct 26 2017 adrian.glaubitz@suse.com
- Update to version 20171025 (bsc#1064356, bsc#1065308)
  + Add apt configuration to prevent auto-removal of Google packages.
  + Rename set_hostname to prevent naming conflicts.
  + Remove logging when checking OS Login status.
- From version 20171019
  + Support the enable-oslogin metadata key for activating OS Login.
  + Improve packaging to restart services.
  + OS Login is available in Beta.
  + Add status option to the OS Login control file.
- From version 20171006
  + Fix system hang during VM shutdown.
  + JSON parser accepts string types for int64 values.
- From version 20170921
  + JSON parser casts uid and gid to unsigned integers.
- From version 20170914
  + Remove fstab barrier options in EL 7.
  + Use curl to download metadata script files for SSL certificate validation.
  + Use netifaces for retrieving MAC address names if the import exists.
* Tue Sep 19 2017 rjschwei@suse.com
- Ship the udevrules with the -init package only
* Tue Sep 12 2017 rjschwei@suse.com
- Fix baslibs.conf, use package, not files
- Include rpmlintrc and baslibc.conf as source
* Mon Sep 11 2017 rjschwei@suse.com
- Add gcei-scripts-after-reg.patch (bsc#1057671)
* Sat Sep  9 2017 rjschwei@suse.com
- Update to version 20170829 (bsc#1049242, FATE#323757)
  + Support oslogin feature
  + Add rpmlintrc
    ~ We ship pam and nss modules in -oslogin we do not want
    to name the package according to the shared library naming policy
  + Add baslibs.config
    ~ Handle the nss and pam modules provided by oslogin properly
- From version 20170718
  + Allow nologin paths other than /sbin/nologin.
  + Try to download GCS URLs with curl if gsutil is not installed.
  + Fix control scripts to correctly restart sshd and nscd if they exist.
  + Retry HTTP requests if error 500 is received.
  + Move oslogin sudoers directory locations.
- Setup for Python 3 build oSTW and SLE 15
- Source package renamed to google-compute-engine
  + Binary subpackages -init -oslogin
* Fri Jan 13 2017 rjschwei@suse.com
- The startup script attempts a network connection, thus it must
  run after network setup
* Thu Jan 12 2017 rjschwei@suse.com
- Scripts that are one-shot should not be marked as "stop_on_removal"
  as there is no process running (bsc#1017395)
- One-shot scripts should not run with startproc
* Mon Dec 19 2016 rjschwei@suse.com
- Update to version 20161213 (bsc#1015829, bsc#1016372)
  + Remove gcei-handle-failed-open.patch included upstream
  + Remove gcei-handle-missing-gsutil.patch included upstream
  + Forward port gcei-hide-py-deps.patch
  + Improved alias IP support
- From 20161118
  + Add support for alias IPs in the IP forwarding daemon.
  + IP forwarding daemon adds back local routes after network restart.
  + Account daemon removes expired key access without metadata change.
  + Account daemon ignores SSH keys with non-ascii characters.
  + Improved exception handling.
  + Fix for syslog startup on systemd.
  + Add a route to the metadata server to /etc/hosts.
- From 20160930
  + Provide a service to enable network interfaces on boot.
  + Create a common library for inspecting network interfaces.
  + Allow metadata script output that is not UTF-8.
  + Fixed instance config file logic.
  + Fixed accounts management Python 3 compatibility.
  + Fixed IP forwarding Python 3 compatibility.
  + Improved style consistency.
  + Run a service on boot to enable additional network interfaces.
  + Update dhclient-script on EL 6 to fix local routing.
* Sun Oct 30 2016 jengelh@inai.de
- Resolve description inaccuracy
- Call %%service_* just once, but with all args
* Wed Oct 26 2016 rjschwei@suse.com
- Include in SLE 12 and SLE 11 (FATE#321748, FATE#321890, bsc#994943)
* Thu Oct 20 2016 rjschwei@suse.com
- Update gcei-handle-failed-open.patch to match upstream PR
* Wed Oct 19 2016 rjschwei@suse.com
- Add gcei-handle-failed-open.patch
  * Do not exit with a traceback if the sudoers file cannot be written
* Wed Oct 19 2016 rjschwei@suse.com
- Package the rsyslog config unconditionally, rsyslog also available on
  SLE 11
* Wed Oct 19 2016 rjschwei@suse.com
- Add sysvinit scripts for SUSE, upstream scripts are RHEL specific
  * google-accounts-daemon.suse
  * google-clock-skew-daemon.suse
  * google-instance-setup.suse
  * google-ip-forwarding-daemon.suse
  * google-shutdown-scripts.suse
  * google-startup-scripts.suse
* Mon Oct 17 2016 rjschwei@suse.com
- Own the udev directories, fixes issue with SLE 12 build
* Mon Oct 17 2016 rjschwei@suse.com
- Conflict with the previous generation of initialization code.
  * According to upstream and update path is not supported and
    has too many corner cases to reliably work. Thus running instances are
    not expected to upgrade.
* Mon Oct 17 2016 rjschwei@suse.com
- Initial build
  - Version 20160803
